/*
 * Copyright 2016-present Open Networking Laboratory
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.onosproject.nemo.model.operation;

import static org.junit.Assert.assertEquals;

import java.util.ArrayList;
import java.util.List;

import org.junit.Test;
import org.onosproject.nemo.model.common.ParameterName;
import org.onosproject.nemo.model.operation.condition.parameter.definitions.ConditionParameterDefinition;
import org.onosproject.nemo.model.operation.condition.parameter.definitions.ConditionParameterDefinitionBuilder;

import com.google.common.testing.EqualsTester;

/**
 * Unit tests for {@link ConditionParameterDefinitionsBuilder}.
 */
public class ConditionParameterDefinitionsBuilderTest {

    private static final String DEFINITIONS_ERROR_MSG = "definitions not set";
    private static final String NAME = "SomeName";
    private static final String OTHER_NAME = "OtherName";

    private List<ConditionParameterDefinition> definitionList;
    private List<ConditionParameterDefinition> otherList;
    private ConditionParameterDefinitions definitions;
    private ConditionParameterDefinitions copy;
    private ConditionParameterDefinitions diff;
    private ParameterName name;
    private ParameterName otherName;
    private ConditionParameterDefinition definition;
    private ConditionParameterDefinition otherdefinition;

    @Test
    public void buildConditionParameterDefinitionWithParameterDefinitions() {
        definitionList = new ArrayList<>();
        definitions = new ConditionParameterDefinitionsBuilder()
                .conditionParameterDefinitions(definitionList)
                .build();
        assertEquals(DEFINITIONS_ERROR_MSG, definitionList,
                definitions.getDefinitions());
    }

    @Test
    public void fromConditionParameterDefinitionWithParameterDefinitions() {
        definitionList = new ArrayList<>();
        definitions = new ConditionParameterDefinitionsBuilder()
                .conditionParameterDefinitions(definitionList)
                .build();
        copy = new ConditionParameterDefinitionsBuilder(definitions).build();
        assertEquals(DEFINITIONS_ERROR_MSG, definitionList,
                copy.getDefinitions());
    }

    @Test
    public void equality() {
        definitionList = new ArrayList<>();
        definitions = new ConditionParameterDefinitionsBuilder()
                .conditionParameterDefinitions(definitionList)
                .build();
        otherList = new ArrayList<>();
        otherList.add(null);
        copy = new ConditionParameterDefinitionsBuilder()
                .conditionParameterDefinitions(definitionList)
                .build();
        diff = new ConditionParameterDefinitionsBuilder()
                .conditionParameterDefinitions(otherList)
                .build();
        new EqualsTester()
                .addEqualityGroup(definitions, copy)
                .addEqualityGroup(diff)
                .testEquals();
    }

    @Test
    public void equalityTwo() {
        name = new ParameterName(NAME);
        otherName = new ParameterName(OTHER_NAME);
        definition = new ConditionParameterDefinitionBuilder()
                .parameterName(name)
                .build();
        definitionList = new ArrayList<>();
        definitionList.add(definition);
        definitions = new ConditionParameterDefinitionsBuilder()
                .conditionParameterDefinitions(definitionList)
                .build();
        otherList = new ArrayList<>();
        otherdefinition = new ConditionParameterDefinitionBuilder()
                .parameterName(otherName)
                .build();
        otherList.add(otherdefinition);
        copy = new ConditionParameterDefinitionsBuilder()
                .conditionParameterDefinitions(definitionList)
                .build();
        diff = new ConditionParameterDefinitionsBuilder()
                .conditionParameterDefinitions(otherList)
                .build();
        new EqualsTester()
                .addEqualityGroup(definitions, copy)
                .addEqualityGroup(diff)
                .testEquals();
    }
}